package com.google.android.apps.work.dpcsupport;

import android.accounts.Account;
import android.accounts.AccountManager;
import android.content.Context;
import android.support.annotation.VisibleForTesting;
import android.support.annotation.WorkerThread;
import android.util.Log;
import com.google.android.apps.work.dpcsupport.WorkAccountsRemovedCallback.Error;
import com.google.android.gms.auth.GoogleAuthUtil;

/** Removes all Android for Work accounts on the device. */
public class WorkAccountRemover {
  private final Context context;

  private WorkAccountsRemovedCallback callback;
  private final WorkAccountApiHelper workAccountApiHelper;

  WorkAccountRemover(Context context) {
    this(context, new WorkAccountApiHelper(context));
  }

  @VisibleForTesting
  WorkAccountRemover(Context context, WorkAccountApiHelper workAccountApiHelper) {
    this.context = context;
    this.workAccountApiHelper = workAccountApiHelper;
  }

  @WorkerThread
  void removeAllWorkAccounts(final WorkAccountsRemovedCallback callback) {
    this.callback = callback;

    PlayServicesAndStoreUtils versionUtils = new PlayServicesAndStoreUtils(context);
    if (!versionUtils.isPlayServicesInstalled()) {
      Log.e(Constants.TAG, "Play Services not installed.");
      fail(Error.FAILED_PRECONDITION);
      return;
    }
    if (!versionUtils.isPlayServicesVersionCompatible()) {
      Log.e(
          Constants.TAG,
          "Play Services not up to date. "
              + "Call ensureWorkingEnvironment before removing accounts.");
      fail(Error.FAILED_PRECONDITION);
      return;
    }

    // List work accounts
    AccountManager accountManager = AccountManager.get(context);
    Account[] workAccountsOnDevice =
        accountManager.getAccountsByType(GoogleAuthUtil.WORK_ACCOUNT_TYPE);

    boolean result = removeWorkAccounts(workAccountsOnDevice);
    if (result) {
      succeed();
    } else {
      fail(Error.EXCEPTION_REMOVING_ACCOUNT);
    }
  }

  private boolean removeWorkAccounts(Account[] workAccounts) {
    if (workAccounts.length == 0) {
      return true;
    }
    // Remove all work accounts. If we fail to remove one account we still try to remove the other
    // accounts before returning an error.
    boolean result = true;
    for (Account workAccount : workAccounts) {
      boolean accountResult = workAccountApiHelper.removeWorkAccount(workAccount);
      result &= accountResult;
    }

    return result;
  }

  private void succeed() {
    callback.onSuccess();
  }

  private void fail(final Error error) {
    callback.onFailure(error);
  }
}
